<?php

use Imoje\Payment\Util;
use Imoje\Payment\Notification as SdkNotification;

/**
 * Class ImojeNotificationModuleFrontController
 *
 * @property bool display_column_left
 * @property bool display_column_right
 * @property bool display_footer
 * @property bool display_header
 */
class ImojeNotificationModuleFrontController extends ModuleFrontController
{

	/**
	 * @throws PrestaShopException
	 * @see FrontController::init()
	 */
	public function init()
	{
		$this->display_column_left = false;
		$this->display_column_right = false;
		$this->display_footer = false;
		$this->display_header = false;
		parent::init();
	}

	/**
	 * @throws PrestaShopException
	 * @throws Exception
	 */
	public function process()
	{

		include_once(_PS_MODULE_DIR_ . 'imoje/libraries/payment-core/vendor/autoload.php');

		$notification = new SdkNotification(Configuration::get('IMOJE_SERVICE_ID'), Configuration::get('IMOJE_SERVICE_KEY'));
		$resultCheckRequestNotification = $notification->checkRequest();
		$isDebugMode = Configuration::get('IMOJE_DEBUG_MODE');

		$arrangementAfterIpn = Configuration::get('IMOJE_CREATE_ORDER_ARRANGEMENT');

		if(is_int($resultCheckRequestNotification)) {
			echo SdkNotification::formatResponse(SdkNotification::NS_ERROR,
				$resultCheckRequestNotification,
				$isDebugMode,
				null,
				null,
				$arrangementAfterIpn);
			exit();
		}

		// region if customer made order with blik code on checkout
		if($resultCheckRequestNotification['transaction']['paymentMethod'] === 'blik'
			&& $resultCheckRequestNotification['transaction']['source'] === 'api'
			&& Configuration::get('IMOJE_BLIK_CODE_CHECKOUT')) {

			$this->processIpn($resultCheckRequestNotification, $isDebugMode, $arrangementAfterIpn);

			return;
		}
		// endregion

		if($arrangementAfterIpn) {
			$this->processIpn($resultCheckRequestNotification, $isDebugMode, $arrangementAfterIpn);

			return;
		}

		$this->processCheckout($resultCheckRequestNotification, $isDebugMode, $arrangementAfterIpn);
	}

	/**
	 * @param array $ipn
	 * @param int   $debugMode
	 * @param int   $orderArrangement
	 *
	 * @return void
	 */
	private function processIpn($ipn, $debugMode, $orderArrangement)
	{
		if($ipn['transaction']['status'] !== 'settled') {
			echo SdkNotification::formatResponse(SdkNotification::NS_OK, SdkNotification::NC_ORDER_STATUS_IS_NOT_SETTLED_ORDER_ARRANGEMENT_AFTER_IPN, $debugMode, null, null, $orderArrangement);
			exit;
		}

		$cart = new Cart($ipn['transaction']['orderId']);

		if(!$cart->id) {
			echo SdkNotification::formatResponse(
				SdkNotification::NS_ERROR,
				SdkNotification::NC_CART_NOT_FOUND,
				$debugMode,
				null,
				null,
				$orderArrangement);
			exit;
		}

		if(Order::getOrderByCartId($cart->id)) {
			echo SdkNotification::formatResponse(
				SdkNotification::NS_OK,
				SdkNotification::NC_ORDER_EXISTS_ORDER_ARRANGEMENT_AFTER_IPN,
				$debugMode,
				null,
				null,
				$orderArrangement);
			exit;
		}

		$currencyInfo = Currency::getCurrency($cart->id_currency);

		$cartTotal = $cart->getOrderTotal();

		if(!SdkNotification::checkRequestAmount(
			$ipn,
			Util::convertAmountToFractional($cartTotal),
			$currencyInfo["iso_code"])) {

			echo SdkNotification::formatResponse(
				SdkNotification::NS_ERROR,
				SdkNotification::NC_AMOUNT_NOT_MATCH,
				$debugMode,
				null,
				null,
				$orderArrangement
			);
			exit();
		}

		$customer = new Customer((int) $cart->id_customer);
		$cartId = $cart->id;

		$imoje = new Imoje();
		$imoje->validateOrder($cartId,
			_PS_OS_PAYMENT_,
			$cartTotal,
			$imoje->displayName,
			null,
			[],
			null,
			false,
			$customer->secure_key
		);

		$history = new OrderHistory();
		$history->id_order = intval($ipn['transaction']['orderId']);

		$order = new Order(Order::getOrderByCartId((int) $ipn['transaction']['orderId']));
		$payments = $order->getOrderPaymentCollection()->getResults();

		if(count($payments) > 0) {
			$payments[0]->transaction_id = $ipn['transaction']['id'];
			$payments[0]->update();
		}

		echo SdkNotification::formatResponse(
			SdkNotification::NS_OK,
			SdkNotification::NC_OK,
			$debugMode,
			null,
			_PS_OS_PAYMENT_,
			$orderArrangement
		);
		exit();
	}

	/**
	 * @param array $ipn
	 * @param int   $debugMode
	 * @param int   $orderArrangement
	 *
	 * @return void
	 */
	private function processCheckout($ipn, $debugMode, $orderArrangement)
	{
		$order = new Order((int) $ipn['transaction']['orderId']);
		$orderId = $order->id;

		if(!$orderId) {
			echo SdkNotification::formatResponse(
				SdkNotification::NS_ERROR,
				SdkNotification::NC_ORDER_NOT_FOUND,
				$debugMode,
				null,
				null,
				$orderArrangement
			);
			exit();
		}

		$currentOrderState = $order->current_state;

		switch($ipn['transaction']['status']) {
			case 'settled':
				$stateToChange = _PS_OS_PAYMENT_;
				break;
			case 'rejected':
				$stateToChange = _PS_OS_ERROR_;
				break;
			default:
				echo SdkNotification::formatResponse(
					SdkNotification::NS_OK,
					SdkNotification::NC_ORDER_STATUS_NOT_CHANGED,
					$debugMode,
					$currentOrderState,
					null,
					$orderArrangement
				);
				exit();
		}

		if($currentOrderState === _PS_OS_REFUND_) {
			echo SdkNotification::formatResponse(
				SdkNotification::NS_OK,
				SdkNotification::NC_ORDER_STATUS_NOT_CHANGED,
				$debugMode,
				$currentOrderState,
				null,
				$orderArrangement
			);
			exit();
		}

		$currencyInfo = Currency::getCurrency($order->id_currency);

		if(!SdkNotification::checkRequestAmount(
			$ipn,
			Util::convertAmountToFractional(
				round($order->total_paid, 2)),
			$currencyInfo["iso_code"]
		)) {

			echo SdkNotification::formatResponse(
				SdkNotification::NS_ERROR,
				SdkNotification::NC_AMOUNT_NOT_MATCH,
				$debugMode,
				$currentOrderState,
				null,
				$orderArrangement
			);

			exit();
		}

		$history = new OrderHistory();
		$history->id_order = $orderId;
		$history->changeIdOrderState($stateToChange, $orderId);
		$history->addWithemail(true);

		$payments = $order->getOrderPaymentCollection()->getResults();

		if(count($payments) > 0) {
			$payments[0]->transaction_id = $ipn['transaction']['id'];
			$payments[0]->update();
		}

		echo SdkNotification::formatResponse(
			SdkNotification::NS_OK,
			SdkNotification::NC_OK,
			$debugMode,
			$currentOrderState,
			$stateToChange,
			$orderArrangement
		);

		exit;
	}
}
